@@ -1,9 +1,8 @@
 use super::Optimizer;
 use crate::mode::Mode;
 use swc_atoms::js_word;
-use swc_common::Spanned;
+use swc_common::{util::take::Take, Spanned};
 use swc_ecma_ast::*;
-use swc_ecma_transforms_base::ext::MapWithMut;
 use swc_ecma_utils::{ident::IdentLike, ExprExt, Value::Known};
 
 impl<M> Optimizer<'_, M>
