@@ -1,7 +1,7 @@
 use super::Pure;
 use crate::mode::Mode;
+use swc_common::util::take::Take;
 use swc_ecma_ast::*;
-use swc_ecma_transforms_base::ext::MapWithMut;
 
 impl<M> Pure<'_, M>
 where
